import 'dart:async';
import 'dart:io';

import 'package:flutter/services.dart';

/// You can use shortcuts like pressing home and back button programatically
/// by calling home ( ) and back ( ) functions respectively.
///
/// You can also control volume keys by calling volUp ( ) and volDown ( )
/// to press volume Up button and volume Down button respectively.
///
/// You can change the orientation of your app by calling functions
/// orientLandscape ( ) or orientPortrait ( ) to turn the app into
/// landscape and portrait mode respectively.
///
/// You can toggle wifi and bluetooth as well using wifi ( ) and bluetooth ( )
/// functions respectively
class SystemShortcuts {
  static const MethodChannel _channel = MethodChannel('lecle_system_shortcuts');

  /// Press home button programatically .
  ///
  /// Implemetation :-
  ///
  /// await SystemShortcuts.home ( );
  static Future<void> home() async {
    await _channel.invokeMethod('home');
  }

  /// Press back button programatically .
  ///
  /// Implemetation :-
  ///
  /// await SystemShortcuts.back ( );
  static Future<void> back() async {
    await _channel.invokeMethod('back');
  }

  /// Press Volume Down button programatically .
  ///
  /// Implemetation :-
  ///
  /// await SystemShortcuts.volDown ( );
  static Future<void> volDown(
      {AudioManger? streamType, bool? showVolumeUiFlag}) async {
    await _channel.invokeMethod(
      'volDown',
      {
        'streamType': _getStreamInt(streamType),
        'showVolumeUiFlag': showVolumeUiFlag ?? false ? 1 : 0,
      },
    );
  }

  /// Press Volume Up button programatically .
  ///
  /// Implemetation :-
  ///
  /// await SystemShortcuts.volUp ( );
  static Future<void> volUp(
      {AudioManger? streamType, bool? showVolumeUiFlag}) async {
    await _channel.invokeMethod(
      'volUp',
      {
        'streamType': _getStreamInt(streamType),
        'showVolumeUiFlag': showVolumeUiFlag ?? false ? 1 : 0,
      },
    );
  }

  /// Change app orientation to landscape mode
  ///
  /// Implemetation :-
  ///
  /// await SystemShortcuts.orientLandscape ( );
  static Future<void> orientLandscape() async {
    await _channel.invokeMethod('orientLandscape');
  }

  /// Change app orientation to Portrait mode
  ///
  /// Implemetation :-
  ///
  /// await SystemShortcuts.orientPortrait ( );
  static Future<void> orientPortrait() async {
    await _channel.invokeMethod('orientPortrait');
  }

  /// Toggle Wifi.
  ///
  /// If it is already turned on wifi ( ) will turn it off
  /// else it'll turn it on
  static Future<void> wifi() async {
    if (Platform.isIOS) {
    } else {
      await _channel.invokeMethod('wifi');
    }
  }

  /// Return true if the wifi is alreay turned on.
  ///
  /// Return false if the wifi is turned off.
  static Future<bool?> get checkWifi async {
    bool? b = await _channel.invokeMethod('checkWifi');
    return b;
  }

  /// Toggle bluetooth.
  ///
  /// If it is already turned on bluetooth ( ) will turn it off
  /// else it'll turn it on
  static Future<void> bluetooth() async {
    if (Platform.isIOS) {
    } else {
      await _channel.invokeMethod('bluetooth');
    }
  }

  /// Return true if the bluetooth is alreay turned on.
  ///
  /// Return false if the bluetooth is turned off.
  static Future<bool?> get checkBluetooth async {
    bool? b = await _channel.invokeMethod('checkBluetooth');
    return b;
  }

  /// Get the stream type int value for invokeChannel
  static int? _getStreamInt(AudioManger? streamType) {
    switch (streamType) {
      case AudioManger.streamVoiceCall:
        return 0;
      case AudioManger.streamSystem:
        return 1;
      case AudioManger.streamRing:
        return 2;
      case AudioManger.streamMusic:
        return 3;
      case AudioManger.streamAlarm:
        return 4;
      case AudioManger.streamNotification:
        return 5;
      default:
        return null;
    }
  }
}

/// AudioManger streams control the type of volume the getVol and setVol function will control.
///
/// **Use for Android device only, because in iOS we can adjust only the system stream**
enum AudioManger {
  /// Controls the Voice Call volume
  streamVoiceCall,

  /// Controls the system volume
  streamSystem,

  /// Controls the ringer volume
  streamRing,

  /// Controls the media volume
  streamMusic,

  /// Controls the alarm volume
  streamAlarm,

  /// Controls the notification volume
  streamNotification
}
